//
// Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
// Ministerpräsidenten des Landes Schleswig-Holstein
// Staatskanzlei
// Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
//
// Lizenziert unter der EUPL, Version 1.2 oder - sobald
// diese von der Europäischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie dürfen dieses Werk ausschließlich gemäß
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
// https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
// ausdrücklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschränkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package server

import (
	"context"
	pb "fachstellen-proxy/gen/go"
	"fachstellen-proxy/internal/config"
	"fachstellen-proxy/internal/mock"
	"fmt"
	"github.com/stretchr/testify/assert"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
	"google.golang.org/grpc/metadata"
	"net"
	"testing"
	"time"
)

func TestStartCollaborationRouter(t *testing.T) {
	setUpGrpcRouterEnv := func() (pb.CollaborationServiceClient, func()) {
		mock.SetUpGrpcServer()
		SetUpCollaborationRouter()

		conn, err := grpc.NewClient("localhost:50051", grpc.WithTransportCredentials(insecure.NewCredentials()))
		assert.NoError(t, err)

		cleanUp := func() {
			conn.Close()
		}
		client := pb.NewCollaborationServiceClient(conn)

		return client, cleanUp
	}

	t.Run("should start collaboration router", func(t *testing.T) {
		SetUpCollaborationRouter()

		conn, err := net.DialTimeout("tcp", "localhost:50051", 2*time.Second)

		assert.NoError(t, err)

		conn.Close()
	})

	t.Run("should have no error", func(t *testing.T) {
		client, cleanUp := setUpGrpcRouterEnv()
		defer cleanUp()

		md := metadata.New(map[string]string{GrpcAddressMetadata: "localhost"})
		ctx := metadata.NewOutgoingContext(context.Background(), md)
		resp, err := client.FindVorgang(ctx, &pb.GrpcFindVorgangRequest{VorgangId: "testVorgangId", SamlToken: "testSamlToken"})

		assert.NoError(t, err)
		assert.NotNil(t, resp)
	})

	t.Run("should have error", func(t *testing.T) {
		client, cleanUp := setUpGrpcRouterEnv()
		defer cleanUp()

		md := metadata.New(map[string]string{GrpcAddressMetadata: "localhost"})
		ctx := metadata.NewOutgoingContext(context.Background(), md)
		resp, err := client.FindVorgang(ctx, &pb.GrpcFindVorgangRequest{})

		assert.Error(t, err)
		assert.Contains(t, err.Error(), "vorgangId is missing")
		assert.Nil(t, resp)
	})
}

func TestGetCollaborationServerUrl(t *testing.T) {
	t.Run("should have configured server url", func(t *testing.T) {
		c := config.Config{}
		c.Grpc.Collaboration.Server.Port = 99999

		result := GetCollaborationServerUrl("localhost", c)

		assert.Equal(t, "localhost:99999", result)
	})

	t.Run("should have mock server url", func(t *testing.T) {
		c := config.Config{}
		c.Grpc.Mock = true

		result := GetCollaborationServerUrl("localhost", c)

		assert.Equal(t, fmt.Sprintf("localhost:%d", mock.GrpcMockPort), result)
	})
}
