//
// Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
// Ministerpräsidenten des Landes Schleswig-Holstein
// Staatskanzlei
// Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
//
// Lizenziert unter der EUPL, Version 1.2 oder - sobald
// diese von der Europäischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie dürfen dieses Werk ausschließlich gemäß
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
// https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
// ausdrücklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschränkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package server

import (
	"bytes"
	"context"
	log "github.com/sirupsen/logrus"
	pb "fachstellen-proxy/gen/go"
	"fmt"
	"github.com/grpc-ecosystem/grpc-gateway/v2/runtime"
	"google.golang.org/grpc"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/credentials/insecure"
	"google.golang.org/grpc/status"
	"io"
	"net/http"
)

const (
	fileIdParam          = "id"
	samlTokenParam       = "samlToken"
	contentTypeHeaderKey = "Content-Type"
)

func RegisterHomeEndpoint(mux *runtime.ServeMux) {
	err := mux.HandlePath("GET", "/", func(w http.ResponseWriter, r *http.Request, pathParams map[string]string) {
		defer func() {
			if err := recover(); err != nil {
				log.Error(fmt.Sprintf("failed to recover: %v", err))
				http.Error(w, fmt.Sprintf("failed to recover: %v", err), http.StatusInternalServerError)
			}
		}()
	})

	if err != nil {
		log.Fatal(fmt.Sprintf("failed to register home endpoint: %v", err))
	}
}

func RegisterFachstelleRegistrationEndpoints(ctx context.Context, mux *runtime.ServeMux, grpcUrl string) {
	opts := []grpc.DialOption{grpc.WithTransportCredentials(insecure.NewCredentials())}
	err := pb.RegisterFachstelleRegistrationServiceHandlerFromEndpoint(ctx, mux, grpcUrl, opts)
	if err != nil {
		log.Fatal(fmt.Sprintf("failed to register fachstelle registration endpoints: %v", err))
	}
}

func RegisterCollaborationEndpoints(ctx context.Context, mux *runtime.ServeMux, grpcUrl string) {
	opts := []grpc.DialOption{grpc.WithTransportCredentials(insecure.NewCredentials())}
	err := pb.RegisterCollaborationServiceHandlerFromEndpoint(ctx, mux, grpcUrl, opts)
	if err != nil {
		log.Fatal(fmt.Sprintf("failed to register collaboration endpoints: %v", err))
	}
}

func RegisterGetFileEndpoint(mux *runtime.ServeMux) {
	err := mux.HandlePath("GET", "/api/file/{"+fileIdParam+"}/{"+samlTokenParam+"}", getFileHandler)

	if err != nil {
		log.Fatal(fmt.Sprintf("failed to register get file endpoint: %v", err))
	}
}

func getFileHandler(w http.ResponseWriter, r *http.Request, vars map[string]string) {
	fileBuffer, err := fetchFileFromGrpc(r.Context(), vars[fileIdParam], vars[samlTokenParam], r.Header.Get(GrpcAddressHeader))
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	w.Header().Set(contentTypeHeaderKey, "application/octet-stream")
	w.WriteHeader(http.StatusOK)

	_, err = fileBuffer.WriteTo(w)
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
	}
}

func fetchFileFromGrpc(ctx context.Context, fileId string, samlToken string, grpcAddress string) (*bytes.Buffer, error) {
	var fileBuffer bytes.Buffer

	target := GetCollaborationServerUrl(grpcAddress, conf)
	conn, err := grpc.NewClient(target, grpc.WithTransportCredentials(insecure.NewCredentials()))
	if err != nil {
		return nil, fmt.Errorf("gRPC connection error")
	}
	defer conn.Close()

	client := pb.NewCollaborationServiceClient(conn)
	req := &pb.GrpcGetFileContentRequest{Id: fileId, SamlToken: samlToken}
	clientStream, err := client.GetFileContent(ctx, req)
	if err != nil {
		return nil, fmt.Errorf("stream creation error")
	}

	for {
		resp, err := clientStream.Recv()
		if err == io.EOF {
			break
		}
		if err != nil {
			return nil, fmt.Errorf("error receiving file chunks")
		}

		_, err = fileBuffer.Write(resp.FileContent)
		if err != nil {
			return nil, fmt.Errorf("error writing file data")
		}
	}

	return &fileBuffer, nil
}

func ErrorHandler(ctx context.Context, mux *runtime.ServeMux, marshaler runtime.Marshaler, w http.ResponseWriter, r *http.Request, err error) {
	st, ok := status.FromError(err)
	if !ok {
		runtime.DefaultHTTPErrorHandler(ctx, mux, marshaler, w, r, err)
		return
	}

	statusCodeMap := map[codes.Code]int{
		codes.InvalidArgument: http.StatusBadRequest,
		codes.AlreadyExists:   http.StatusConflict,
		codes.Unavailable:     http.StatusServiceUnavailable,
	}

	httpStatusCode := http.StatusInternalServerError

	if code, exists := statusCodeMap[st.Code()]; exists {
		httpStatusCode = code
	}

	w.Header().Set("Content-Type", marshaler.ContentType(r))
	w.WriteHeader(httpStatusCode)

	_, writeErr := w.Write([]byte(st.Message()))
	if writeErr != nil {
		log.Fatal(fmt.Sprintf("failed to handle grpc error: %v", writeErr))
	}
}
